module Cubes where

import Data.Boolean.CUDD


isop :: BDD -> BDD -> String -> BDD -> (String , BDD)
isop f_ f argstack argbdd
		-- If f is always 0 or x, no prime cubes can exist and the function returns nothing
	| f_ == false = ("" , false)
		-- If f is always 1 or x, a prime cube has been found and should be returned
	| f  == true  = (argstack, argbdd) 
		-- If f is sometimes 1 and sometimes 0, one or more cubes can be found by further recursion
	| otherwise   =
	let
		-- extract left and right branches of the function
		f0_   = belse f_ 
		f0    = belse f  

		f1_   = bthen f_ 
		f1    = bthen f  
		

		-- turn overlapping true areas into don't - cares 
		f0'_  = f0_ /\ neg f1
		f0'   = f0
		
		f1'_  = f1_ /\ neg f0
		f1'   = f1

		--recursively generate new covers
		--appending the extracted variable onto the stack, (negated for the else branch)
		--returned string is cover, returned BDD is equivalent to cover.

		(isop0 , g0)
			| argstack == "" = isop f0'_ f0' ("~" ++ show (bif f)) (neg (bif f))
			| otherwise      = isop f0'_ f0' (argstack ++ "&" ++ "~" ++ show (bif f)) (argbdd /\ (neg ( bif f)))
		
		
		(isop1 , g1)
			| argstack == "" = isop f1'_ f1' (show (bif f)) (bif f)
			| otherwise      = isop f1'_ f1' (argstack ++ "&" ++ show (bif f)) (argbdd /\ (bif f))
			
		--construct a difference set between the function to be covered, and the then and else covers
		--generating a "dont-care cover":
		
		f0''_ = f0_ /\ neg g0
		f0''  = f0

		f1''_ = f1_ /\ neg g1
		f1''  = f1
		
	
		fdc_    = (f0'' /\ f1''_) \/ (f0''_ /\ f1'')
		fdc     =  f0'' /\ f1''

		
		--recursively generate new cover, leaving out the extracted variable from the stack. 
		(isopdc , gdc) = isop fdc_ fdc argstack argbdd

		-- append together the found covers, inserting or-strings between non-empty covers.
		isoppart = 
			if isop0 == "" || isop1 == "" 
			then isop0 ++ isop1
			else isop0 ++ " | " ++ isop1
		
		isop_total = 
			if isoppart == "" || isopdc == "" 
			then isoppart ++ isopdc
			else isoppart ++ " | " ++ isopdc

		-- calculate a total cover bdd
		g_total = g0 \/ g1 \/ gdc
	in(
		(isop_total, g_total)
	)

