-- Functions that take a list of PNetNode and try to generate cube sets, or print recursively in different formats
module TestPrint where


import Data.Boolean.CUDD
import Text.Printf
import Netlist.Data
import PState.Data
import PState.BDD
import Cubes
import Data.Maybe


bddprint a 
    |a == false = "false"
    |a ==  true = "true"
    |otherwise  = show (bif a) ++ "(" ++ (bddprint $bthen a) ++ ")| ~" ++ show (bif a) ++ "(" ++ (bddprint $belse a) ++ ")"

bddexport a
    |a == false = "false"
    |a ==  true = "true"
    |otherwise  = show (bif a) ++ " /\\ (" ++ (bddexport $bthen a) ++ ") \\/ (neg " ++ show (bif a) ++ " /\\ (" ++ (bddexport $belse a) ++ "))"

bddarglist a
    |l == [] = ""
    |otherwise = foldl (\s x -> s ++ " " ++ x) "" [show x | x <- l]
    where l = support $unmbdd a

unmbdd a = fromMaybe false a

sop_isop a = fst $isop a a "" false

isopList :: [PNetNode] -> IO a0
isopList [] = printf "end!\n" 
isopList (x:xs) = do
    case x of
        Top (a, b, c, d) -> printf "TOP: %d,%d,%d,%d \n" a b c d
        In a b c -> printf "In: %s, %f, %s\n" a b $sop_isop $unmbdd c
        Out a b c d -> printf "Out: %s, %d, %f, %s\n" a b c $sop_isop $unmbdd d
        Local a b c -> printf "Local %s, %d, \n %s \n\n" a b $sop_isop $unmbdd c-- $sop_isop c
        Func a b c d e f g -> printf "Func: %s, %s , %f, %s \n" a c f $sop_isop $unmbdd g -- $sop_isop g 
        State a b c d e f g h -> printf "state: %s, %s, %s, %d, \n %s, \n %s \n\n " a b c d (sop_isop $unmbdd g) (sop_isop $unmbdd h)
    isopList xs


printList :: [PNetNode] -> IO a0
printList [] = printf "end!\n" 
printList (x:xs) = do
    case x of
        Top (a, b, c, d) -> printf "TOP: %d,%d,%d,%d \n" a b c d
        In a b c -> printf "In: %s, %f, %s\n" a b $bddprint $unmbdd c
        Out a b c d -> printf "Out: %s, %d, %f, %s\n" a b c $bddprint $unmbdd d
        Local a b c -> printf "Local %s, %d, \n %s \n\n" a b $bddprint $unmbdd c
        Func a b c d e f g -> printf "Func: %s, %s , %f, %s \n" a c f $bddprint $unmbdd g
        State a b c d e f g h -> printf "state: %s, %s, %s, %d, \n %s, \n %s \n\n " a b c d (bddprint $unmbdd g) (bddprint $unmbdd h)
    printList xs


dumpItem :: PNetNode -> Int -> IO a0
dumpItem x y= do
    case x of
        Top _ ->                 printf ""
        In a _ b ->              printf "f%d %s = %s\n" y (bddarglist b) $bddexport $unmbdd b
        Out a _ _ b ->           printf "f%d %s = %s\n" y (bddarglist b) $bddexport $unmbdd b
        Local a _ b ->           printf "f%d %s = %s\n" y (bddarglist b) $bddexport $unmbdd b
        Func a _ _ _ _ _ b ->    printf "f%d %s = %s\n" y (bddarglist b) $bddexport $unmbdd b
        State a _ _ _ _ _ _ b -> printf "f%d %s = %s\n" y (bddarglist b) $bddexport $unmbdd b

testItem :: PNetNode -> Bool
testItem x = case x of
        Top _ -> False
        otherwise -> True

dumpList :: [PNetNode] -> IO a0
dumpList xs = foldl (>>) (printf "") [dumpItem x y| (x,y) <- zip [a | a <- xs, testItem a] [1 ..]]